#ifndef _STG_TRANSITION_H_
#define _STG_TRANSITION_H_

#include "darray.h"
#include "hashtable.h"

class Transition;
typedef darray <Transition*> TransitionArray;
typedef HashTable <Transition> TransitionHashTable;

#include "stg_signal.h"
#include "stg_node.h"
#include "stg_place.h"

extern const char* PATTERN_DUMMY;
extern const char* PATTERN_TRANSITION;
extern const char* PATTERN_TRANSITION_INDEX;

extern const char SYMBOL_POLARITY_RISING;
extern const char SYMBOL_POLARITY_FALLING;
extern const char SYMBOL_INDEX;


//---------------------------------------------------------------------------
class Transition : public Node
{
  public:
    enum Polarity 
    { 
      POLARITY_UNDEFINED, 
      POLARITY_RISING, 
      POLARITY_FALLING 
    };

    PlaceArray* preset;
    PlaceArray* postset;
    PlaceArray* readset;

    Transition(char* name, SignalArray* signals);
    Transition(Transition& transition);
    ~Transition();

    virtual void setName(char* name);

    void setSignal(Signal* signal);
    Signal* getSignal();

    void setPolarity(Polarity polarity);
    Polarity getPolarity();

    void setIndex(int index);
    int getIndex();
    
  protected:
    SignalArray* signals;

    Signal* signal;
    Polarity polarity;
    int index;

    void genName();
    void parseName();
};

//---------------------------------------------------------------------------
inline Signal* Transition::getSignal()
{
  return this->signal;
}

//---------------------------------------------------------------------------
inline Transition::Polarity Transition::getPolarity()
{
  return this->polarity;
}

//---------------------------------------------------------------------------
inline int Transition::getIndex()
{
  return this->index;
}

#endif
