#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "messages.h"
#include "param.h"
#include "str_utils.h"
#include "example_msgs.h"
#include "example_params.h"

char* _example_program_name;
char* _example_input_file_names;
char* _example_output_file_name;
char* _example_include_file_names;

//---------------------------------------------------------------------------
bool example_process_params(int argc, char** argv)
{
  int i;
  int stdout_num = 1;
  bool result = true;

  example_clear_params();
  char* program_name = strrchr(argv[0], '/');
  if (program_name == NULL)
    _example_program_name = namecopy(argv[0]);
  else
    _example_program_name = namecopy(&program_name[1]);

  if (argc < 2)
    i = 0;
  else
    i = 1;

  while (i > 0)
  {
    // [_example_input_file_names]
    if (argv[i][0] != '-')
    {
      if (_example_input_file_names == NULL)
        _example_input_file_names = namecopy(argv[i]);
      else
      {
        char* input_file_name = name2pattern("%s,%s", _example_input_file_names, argv[i]);
        delete [] _example_input_file_names;
        _example_input_file_names = input_file_name;
      }
      i++;
      if (i >= argc) i = 0;
      continue;
    }
    else
    // -o [_example_output_file_name] <=> --output [_example_output_file_name]
    if (param_string(argc, argv, &i, "o", &_example_output_file_name) ||
        param_string(argc, argv, &i, "-output", &_example_output_file_name))
    {
    }
    else
    // -i [_example_include_file_names] <=> --include [_example_include_file_names]
    if (param_string(argc, argv, &i, "i", &_example_include_file_names) ||
        param_string(argc, argv, &i, "-include", &_example_include_file_names))
    {
    }
    else
    // -h <=> --help
    if (param_bool(argc, argv, &i, "h") ||
        param_bool(argc, argv, &i, "-help"))
    {
      if (i != 0)
        Warning("<%s> must be the only flag",argv[i-1]);
      char* _example_help = example_get_help(_example_program_name);
      fprintf(stderr, _example_help);
      delete [] _example_help;
      result = false;
    }
    else
    // -v <=> --version
    if (param_bool(argc, argv, &i, "v") ||
        param_bool(argc, argv, &i, "-vesion"))
    {
      if (i != 0)
        Warning("\"%s\" must be the only flag",argv[i-1]);
      char* _example_version = example_get_version(_example_program_name);
      fprintf(stderr, _example_version);
      delete [] _example_version;
      result = false;
    }
    else
    // UNRECOGNIZED FLAG
    {
      Error("unrecognized flag \"%s\"", argv[i]);
      result = false;
    }
  }
  example_correct_params();
  return result;
}

//---------------------------------------------------------------------------
void example_clear_params()
{
  delete [] _example_program_name; _example_program_name = NULL;
  delete [] _example_input_file_names; _example_input_file_names = NULL;
  delete [] _example_output_file_name; _example_output_file_name = NULL;
  delete [] _example_include_file_names; _example_include_file_names = NULL;
}

//---------------------------------------------------------------------------
void example_correct_params()
{
  if (_example_program_name == NULL)
    _example_program_name = namecopy(_EXAMPLE_PROGRAM_NAME);
  if (_example_input_file_names == NULL)
    _example_input_file_names = namecopy(_EXAMPLE_INPUT_FILE_NAMES);
  if (_example_output_file_name == NULL)
    _example_output_file_name = namecopy(_EXAMPLE_OUTPUT_FILE_NAME);
  if (_example_include_file_names == NULL)
    _example_include_file_names = namecopy(_EXAMPLE_INCLUDE_FILE_NAMES);
  // if no input file specified read it from standard input
  if (_example_input_file_names == NULL)
    _example_input_file_names = namecopy(NAME_STDIO);
  // if no output file specified write it to standard output
  if (_example_output_file_name == NULL)
    _example_output_file_name = namecopy(NAME_STDIO);

  // count the number of files read from standard input and written to standard output
  int stdin_num = 0;
  int stdout_num = 0;

  stdin_num += namesnamenum(_example_input_file_names, NAME_STDIO);
  stdin_num += namesnamenum(_example_include_file_names, NAME_STDIO);
  stdout_num += namesnamenum(_example_output_file_name, NAME_STDIO);

  if (stdin_num > 1)
    Warning("%d files will be read from standard input", stdin_num);
  if (stdout_num > 1)
    Warning("%d files will be written to standard output", stdout_num);
}
