#include "sv_instance.h"
#include "str_utils.h"
#include "messages.h"

//---------------------------------------------------------------------------
Instance::Instance(int tag, char* name, int index, char* module_name, ConnectionArray* connections) : VerilogObject(tag, name)
{
  this->index = index;
  this->module_name = module_name;
  this->container = NULL;
  this->connections = NULL;
  if (connections)
    this->connections = connections;
}

//---------------------------------------------------------------------------
Instance::Instance(const Instance &instance) : VerilogObject(instance)
{
  this->index = instance.index;
  this->module_name = namecopy(instance.module_name);
  this->container = instance.container;
  this->connections = NULL;
  if (instance.connections)
  {
    this->connections = new ConnectionArray(instance.connections->num);
    for (int i = 0; i < instance.connections->num; i++)
      this->addConnection(new Connection(*instance.connections->arr[i]));
  }
}

//---------------------------------------------------------------------------
Instance::~Instance()
{
  this->clean();
  delete this->connections; this->connections = NULL;
  delete [] module_name; module_name = NULL;
}

//---------------------------------------------------------------------------
void Instance::clean()
{
  if (this->connections)
  {
    this->connections->removeAll(true);
  }
}

//---------------------------------------------------------------------------
void Instance::setIndex(int index)
{
  this->index = index;
}

//---------------------------------------------------------------------------
int Instance::getIndex()
{
  return this->index;
}

//---------------------------------------------------------------------------
void Instance::setModuleName(char* module_name)
{
  delete [] this->module_name;
  this->module_name = module_name;
}

//---------------------------------------------------------------------------
char* Instance::getModuleName()
{
  return this->module_name;
}

//---------------------------------------------------------------------------
void Instance::setContainer(Module* container)
{
  this->container = container;
}

//---------------------------------------------------------------------------
Module* Instance::getContainer()
{
  return this->container;
}

//---------------------------------------------------------------------------
char* Instance::genFullName()
{
  char* name_full = NULL;
  char* name_escape = NULL;

  if (this->isEscape())
    name_escape = name2pattern(PATTERN_FULL_NAME_ESCAPE, this->name);
  else
    name_escape = namecopy(this->name);

  if (this->index < 0)
    name_full = name_escape;
  else
  {
    name_full = name2pattern(PATTERN_INSTANCE_FULL_NAME, name_escape, this->index);
    delete [] name_escape;
  }

  return name_full;
}

//---------------------------------------------------------------------------
char* Instance::genLabelName()
{
  if (this->index < 0)
    return namecopy(this->name);
  else
    return name2pattern(PATTERN_INSTANCE_LABEL, this->name, this->index);
}

//---------------------------------------------------------------------------
void Instance::addConnection(Connection* connection, int ind)
{
  if (!this->connections)
    this->connections = new ConnectionArray(1);
  if (ind < 0)
    this->connections->add(connection);
  else
    this->connections->addAt(ind, connection);
}

//---------------------------------------------------------------------------
void Instance::removeConnection(Connection* connection)
{
  if (this->connections)
  if (connection)
    this->connections->remove(connection, true);
}

//---------------------------------------------------------------------------
Connection* Instance::getConnection(char* connection_name)
{
  if (this->connections)
  {
    Connection* connection_mask = new Connection(TAG_ALL, namecopy(connection_name));
    int connection_ind = connections->find(connection_mask, 0, VerilogObject::compareByMask);
    delete connection_mask;
    if (connection_ind >= 0)
      return connections->arr[connection_ind];
  }
  return NULL;
}

