#ifndef _SV_LIB_H_
#define _SV_LIB_H_

class Lib;

#include <stdio.h>
#include "sv_object.h"
#include "sv_module.h"

// key words of verilog netlist
#define KEY_MODULE                        "module"
#define KEY_ENDMODULE                     "endmodule"
#define KEY_INPUT                         "input"
#define KEY_OUTPUT                        "output"
#define KEY_INOUT                         "inout"
#define KEY_REG                           "reg"
#define KEY_WIRE                          "wire"
#define KEY_PARAMETER                     "parameter"
#define KEY_DELAY                         "#"

//---------------------------------------------------------------------------
class Lib
{
  protected:
    void clean();
    void findHierarchy(ModuleArray* modules, Module* module, int depth);
    void writeModule(FILE* file, Module* module_cur, int tag_ignore = TAG_OBJ_HIDDEN);

  public:
    ModuleArray* modules;
    ModuleHashTable* module_hash_table;

    Lib(ModuleArray* modules = NULL);
    ~Lib();

    void readModules(char* file_names);
    void writeModules(char* file_name, ModuleArray* modules);
    bool addModule(Module* module, int position = -1);
    void addModules(ModuleArray* modules);
    Module* getModule(char* module_name);

    void calcConnections(ModuleArray* modules);
    void sortConnections(ModuleArray* modules);

    void calcHierarchy(ModuleArray* modules = NULL);
    int getMaxDepth(ModuleArray* modules);
};

#endif

