// Verilog model for vme_read_sol_20 
// Change to negative logic on the purpose of saved area and increased speed
// Use gates in ams.v (corresponding verilog library) including complex gates
// No Set/Reset pins required

module vme_read_sol_20_net (
    DSr,
    LDTACK,
    D,
    LDS,
    DTACK
);

input DSr;
input LDTACK;

output D;
output LDS;
output DTACK;


// Functions not mapped into library gates:
// ----------------------------------------

// Inverted inputs
IN1 I1 (DSr, nDSr);
IN1 I_0 (D, nD);
IN1 I_3 (csc0, ncsc0);

// Equation: D = LDTACK DSr csc0'
NA3 I2 (LDTACK, DSr, ncsc0, D_n);
IN1 I3 (D_n, D);

// Equation: LDS = DSr csc0' + D
AN21 I4 (DSr, ncsc0, D, LDS_n);
IN1 I5 (LDS_n, LDS);

// Equation: DTACK = DSr' csc0' + D
AN21 I6 (nDSr, ncsc0, D, DTACK_n);
IN1 I7 (DTACK_n, DTACK);

// Equation: csc0 = D' (LDTACK csc0 + DSr')
AN32 I8 (nD, LDTACK, csc0, nD, nDSr, csc0_n);
IN1 I9 (csc0_n, csc0);

// signal values at the initial state:
//     !DSr !LDTACK !D !LDS !DTACK csc0
endmodule
