#ifndef LEFFORT_H
#define LEFFORT_H


#include <fstream.h>
#include "sv_lib.h"
#include "sv_circuit.h"
#include "le_graph.h"
#include "le_trace.h"


//////////////////////////////////////////
//parameters

#define SEPERATOR       	"_"
#define REF_INVERTER    	"IN1"
#define REF_INVERTER_PIN 	"A"

//input LE parameter
#define D1_PRAC_RISE      "d1_rise"
#define D2_PRAC_RISE      "d2_rise"
#define D1_PRAC_FALL      "d1_fall"
#define D2_PRAC_FALL      "d2_fall"

#define C_PRAC_IN      		"c_in_prac"
#define C_THEO_IN      		"c_in_theo"

//logical efforts
#define G_PRAC_RISE    		"g_rise"
#define G_PRAC_FALL    		"g_fall"
#define G_THEO    				"g"

//parasitics
#define P_PRAC_RISE    		"p_rise"
#define P_PRAC_FALL    		"p_fall"
#define P_THEO    				"p"


//output capacitance corresponding to logical effort parameters
const double C1_PRAC_OUT = 0.015;		//0.015pF
const double C2_PRAC_OUT = 0.15;	 	//0.15pF

//slope parameters

//tau = alpha*slope_in + beta
const double ALPHA_RISE = 0.048;
const double ALPHA_FALL = 0.01;
const double BETA_RISE = 0.033;
const double BETA_FALL = 0.028;

//slope_out = a*h + b*slope_in + c
const double A_RISE = 0.087;
const double A_FALL = 0.057;
const double B_RISE = 0.221;
const double B_FALL = 0.25;
const double C_RISE = 0.062;
const double C_FALL = 0.02;

//initial slope
const double SLOPE_INITIAL = 0.3;



extern void read_parameters(char *, Lib *);


class LEffort
{
	public:
		//constructor & destructor
		LEffort();
		LEffort(const LEffort &);
		~LEffort();

		
		//	public accessor methods
		//==========================
		
		darray<Trace*> * getTraces() const { return traces; }
		
    //	public methods
    //==================

		void readVerilog(char *, char *);	
		void readParameters(char *);
		void readSTG(char *);
		void readLEparameters(char *);


		void extractTraces();	
		void calculateDelay();

		void printTraces();
		void printTraces(char *);


    //overload operators
		friend ostream& operator << (ostream &, const LEffort &);


		//	public member variables
		//===========================
		

		

	private:

		//	private methods
		//===================

		//set traces
		void setConnectionTraces();
		Connection * findConnection(char *);
		void extractPaths();
		void extractPath(Connection *, Connection *, int);
		bool isNameEqual(Net *, Connection *);
		bool isTracesEnd(Net *);
		Connection * getInstancesOutput(Instance *);
		void resetTags();

		//set parammeter
		void setParameters();
		void setRefParameters();
		void setParameters(Module *, char *);
		void setParameter(Module *, char *, char *, double);
		double getParameter(Module *, char *, char *);

		//calculate delay
		void setDelay(Element *, Element *, bool);
		double getLogicalEffort(bool, bool, Module *, char *);
		double getElectricalEffort(bool, Module *, char *, Net *);
		double getParasiticDelay(bool, bool, Module *, char *);
		double getOutputSlope(bool, double, double, double);
		double getTauSlope(bool, double);
		double calculateCout(bool, Net *);
		double getCin(bool, Net *, Instance *);
		Connection * getInstancesInputConnection(Instance *, Connection *);


		//print traces
		void printTrace(Trace *);
		void printTrace(Trace *,ofstream &);
		void printPath(Path *);
		void printPath(Path *,ofstream &);


    //	private member variables
		//===========================

		Lib *myLib;
		Circuit *myCircuit;
		Module *myModule;
		Graph *myGraph;

		darray<Trace*> * traces;
		darray<Path*> *paths;

		Trace *myTrace;
		Path *myPath;

		double tau_ref; //tau_rise of reference inverter
		double tau_ref_rise;
		double tau_ref_fall;
		double c_in_prac_inv;
		double c_in_theo_inv;
    
};

#endif

