#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "messages.h"
#include "param.h"
#include "str_utils.h"
#include "le_msgs.h"
#include "le_params.h"

char* _stg_program_name;
char* _stg_input_file_names;
char* _stg_output_file_name;


char* _sv_program_name;
char* _sv_input_file_names;
char* _sv_output_file_name;
char* _sv_include_file_names;

//---------------------------------------------------------------------------
bool stg_process_params(int argc, char **argv)
{
  int i;
  int stdout_num = 1;
  bool result = true;

  stg_clear_params();
  char* program_name = strrchr(argv[0], '/');
  if (program_name == NULL)
    _stg_program_name = namecopy(argv[0]);
  else
    _stg_program_name = namecopy(&program_name[1]);

  if (argc < 2)
    i = 0;
  else
    i = 1;

  while (i > 0)
  {
    // [_stg_input_file_name]
    if (argv[i][0] != '-')
    {
      if (_stg_input_file_names == NULL)
        _stg_input_file_names = namecopy(argv[i]);
      else
      {
        char* input_file_name = name2pattern("%s,%s", _stg_input_file_names, argv[i]);
        delete [] _stg_input_file_names;
        _stg_input_file_names = input_file_name;
      }
      i++;
      if (i >= argc) i = 0;
      continue;
    }
    else
    // -o [_stg_output_file_name] <=> --output [_stg_output_file_name]
    if (param_string(argc, argv, &i, "o", &_stg_output_file_name) ||
        param_string(argc, argv, &i, "-output", &_stg_output_file_name))
    {
    }
    else
    // -h <=> --help
    if (param_bool(argc, argv, &i, "h") ||
        param_bool(argc, argv, &i, "-help"))
    {
      if (i != 0)
        Warning("<%s> must be the only flag",argv[i-1]);
      char* _stg_help = stg_get_help(_stg_program_name);
      fprintf(stderr, _stg_help);
      delete [] _stg_help;
      result = false;
    }
    else
    // -v <=> --version
    if (param_bool(argc, argv, &i, "v") ||
        param_bool(argc, argv, &i, "-vesion"))
    {
      if (i != 0)
        Warning("\"%s\" must be the only flag",argv[i-1]);
      char* _stg_version = stg_get_version(_stg_program_name);
      fprintf(stderr, _stg_version);
      delete [] _stg_version;
      result = false;
    }
    else
    // UNRECOGNIZED FLAG
    {
      Error("unrecognized flag \"%s\"", argv[i]);
      result = false;
    }
  }
  stg_correct_params();
  return result;
}

//---------------------------------------------------------------------------
void stg_clear_params()
{
  delete [] _stg_program_name; _stg_program_name = NULL;
  delete [] _stg_input_file_names; _stg_input_file_names = NULL;
  delete [] _stg_output_file_name; _stg_output_file_name = NULL;
}

//---------------------------------------------------------------------------
void stg_correct_params()
{
  if (_stg_program_name == NULL)
    _stg_program_name = namecopy(_STG_PROGRAM_NAME);
  if (_stg_input_file_names == NULL)
    _stg_input_file_names = namecopy(_STG_INPUT_FILE_NAMES);
  if (_stg_output_file_name == NULL)
    _stg_output_file_name = namecopy(_STG_OUTPUT_FILE_NAME);
  // if no input file specified read it from standard input
  if (_stg_input_file_names == NULL)
    _stg_input_file_names = namecopy(NAME_STDIO);
  // if no output file specified write it to standard output
  if (_stg_output_file_name == NULL)
    _stg_output_file_name = namecopy(NAME_STDIO);

  // count the number of files read from standard input and written to standard output
  int stdin_num = 0;
  int stdout_num = 0;

  stdin_num += namesnamenum(_stg_input_file_names, NAME_STDIO);
  stdout_num += namesnamenum(_stg_output_file_name, NAME_STDIO);

  if (stdin_num > 1)
    Warning("%d files will be read from standard input", stdin_num);
  if (stdout_num > 1)
    Warning("%d files will be written to standard output", stdout_num);
}















//---------------------------------------------------------------------------
bool sv_process_params(int argc, char** argv)
{
  int i;
  int stdout_num = 1;
  bool result = true;

  sv_clear_params();
  char* program_name = strrchr(argv[0], '/');
  if (program_name == NULL)
    _sv_program_name = namecopy(argv[0]);
  else
    _sv_program_name = namecopy(&program_name[1]);

  if (argc < 2)
    i = 0;
  else
    i = 1;

  while (i > 0)
  {
    // [_sv_input_file_names]
    if (argv[i][0] != '-')
    {
      if (_sv_input_file_names == NULL)
        _sv_input_file_names = namecopy(argv[i]);
      else
      {
        char* input_file_name = name2pattern("%s,%s", _sv_input_file_names, argv[i]);
        delete [] _sv_input_file_names;
        _sv_input_file_names = input_file_name;
      }
      i++;
      if (i >= argc) i = 0;
      continue;
    }
    else
    // -o [_sv_output_file_name] <=> --output [_sv_output_file_name]
    if (param_string(argc, argv, &i, "o", &_sv_output_file_name) ||
        param_string(argc, argv, &i, "-output", &_sv_output_file_name))
    {
    }
    else
    // -i [_sv_include_file_names] <=> --include [_sv_include_file_names]
    if (param_string(argc, argv, &i, "i", &_sv_include_file_names) ||
        param_string(argc, argv, &i, "-include", &_sv_include_file_names))
    {
    }
    else
    // -h <=> --help
    if (param_bool(argc, argv, &i, "h") ||
        param_bool(argc, argv, &i, "-help"))
    {
      if (i != 0)
        Warning("<%s> must be the only flag",argv[i-1]);
      char* _sv_help = sv_get_help(_sv_program_name);
      fprintf(stderr, _sv_help);
      delete [] _sv_help;
      result = false;
    }
    else
    // -v <=> --version
    if (param_bool(argc, argv, &i, "v") ||
        param_bool(argc, argv, &i, "-vesion"))
    {
      if (i != 0)
        Warning("\"%s\" must be the only flag",argv[i-1]);
      char* _sv_version = sv_get_version(_sv_program_name);
      fprintf(stderr, _sv_version);
      delete [] _sv_version;
      result = false;
    }
    else
    // UNRECOGNIZED FLAG
    {
      Error("unrecognized flag \"%s\"", argv[i]);
      result = false;
    }
  }
  sv_correct_params();
  return result;
}

//---------------------------------------------------------------------------
void sv_clear_params()
{
  delete [] _sv_program_name; _sv_program_name = NULL;
  delete [] _sv_input_file_names; _sv_input_file_names = NULL;
  delete [] _sv_output_file_name; _sv_output_file_name = NULL;
  delete [] _sv_include_file_names; _sv_include_file_names = NULL;
}

//---------------------------------------------------------------------------
void sv_correct_params()
{
  if (_sv_program_name == NULL)
    _sv_program_name = namecopy(_SV_PROGRAM_NAME);
  if (_sv_input_file_names == NULL)
    _sv_input_file_names = namecopy(_SV_INPUT_FILE_NAMES);
  if (_sv_output_file_name == NULL)
    _sv_output_file_name = namecopy(_SV_OUTPUT_FILE_NAME);
  if (_sv_include_file_names == NULL)
    _sv_include_file_names = namecopy(_SV_INCLUDE_FILE_NAMES);
  // if no input file specified read it from standard input
  if (_sv_input_file_names == NULL)
    _sv_input_file_names = namecopy(NAME_STDIO);
  // if no output file specified write it to standard output
  if (_sv_output_file_name == NULL)
    _sv_output_file_name = namecopy(NAME_STDIO);

  // count the number of files read from standard input and written to standard output
  int stdin_num = 0;
  int stdout_num = 0;

  stdin_num += namesnamenum(_sv_input_file_names, NAME_STDIO);
  stdin_num += namesnamenum(_sv_include_file_names, NAME_STDIO);
  stdout_num += namesnamenum(_sv_output_file_name, NAME_STDIO);

  if (stdin_num > 1)
    Warning("%d files will be read from standard input", stdin_num);
  if (stdout_num > 1)
    Warning("%d files will be written to standard output", stdout_num);
}


