%{
#include <stdio.h>
#include<stdlib.h>
#include "str_utils.h"
#include "sv_lib.h"
#include "le_leffort.h"


extern FILE *parameter_in;
extern int yyparse();
extern int yylex();
void parameter_error(char *);


Lib *lib = NULL;
Module *module = NULL;
char *pinName = NULL;


%}


%start source

%union
 {
	char * name;
	double value;
 }

%token <name> YYBEGIN
%token <name> YYEND
%token <value> YYNUM
%token <name> YYID




%%

source
: read_parameters
{
	printf("\nparameter reading is complete\n");
}
;


read_parameters
: parameter_list
;

parameter_list
: module_parameter
| module_parameter parameter_list
;

module_parameter
: YYBEGIN '['module_name']' pin_parameter_list YYEND;

module_name
:YYID
{
	module = lib->getModule((char*)$1);
	//printf("\n\n Module name: %s\n ", module->getName());
	if (module == NULL)
	{
		printf("\n Warning: No module in Library!");
		module = new Module;
		module->setName((char*)$1);
	}
	else
		delete []$1;
};


pin_parameter_list
: pin_parameter
| pin_parameter pin_parameter_list
;

pin_parameter
: pin_name d1_rise d2_rise d1_fall d2_fall capacitance_prac capacitance_theo
{
	if (pinName != NULL)
		delete []pinName;
};

pin_name
:YYID
{
	//pinName = new char [strlen((char*)$1) + 1];
	//strcpy(pinName, ((char*)$1));
	pinName = $1;
	//printf("\npin: %s ", pinName);
};


d1_rise
:YYNUM
{
	Parameter *parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", pinName, SEPERATOR, D1_PRAC_RISE), $1);
	module->addParameter(parameter, -1);
	//printf("d1 rise: %f ",$1);
};

d2_rise
:YYNUM
{
	Parameter *parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", pinName, SEPERATOR, D2_PRAC_RISE), $1);
	module->addParameter(parameter, -1);
	//printf("d2 rise: %f ",$1);
};

d1_fall
:YYNUM
{
	Parameter *parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", pinName, SEPERATOR, D1_PRAC_FALL), $1);
	module->addParameter(parameter, -1);
	//printf("d1 fall: %f ",$1);
};

d2_fall
:YYNUM
{
	Parameter *parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", pinName, SEPERATOR, D2_PRAC_FALL), $1);
	module->addParameter(parameter, -1);
	//printf("d2 fall: %f ",$1);
};

capacitance_prac
:YYNUM
{
	Parameter *parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", pinName, SEPERATOR, C_PRAC_IN), $1);
	module->addParameter(parameter, -1);
	//printf("capacitance: %f ",$1);
};


capacitance_theo
:YYNUM
{
	Parameter *parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", pinName, SEPERATOR, C_THEO_IN), $1);
	module->addParameter(parameter, -1);
	//printf("capacitance theo: %f ",$1);
};

%%


void read_parameters(char* filename, Lib *myLib)
{
	lib = myLib;
	parameter_in = fopen(filename,"rb");

	if(!parameter_in)
	{
		fprintf(stderr,"Error could not open file %s\n", filename);
		exit(1);
	}

	yyparse();
	fclose(parameter_in);
}

   



void parameter_error(char *s)
 {
   fprintf(stderr,"%s\n",s);
 }
 

 

