%{
#include "messages.h"
#include "str_utils.h"
#include "stg_net.h"
#include "stg_y.h"

//#define DEBUG 1
#ifdef DEBUG
  #include <stdio.h>
  #define DEBUG_MSG(msg) fprintf(stderr, msg)
#else
  #define DEBUG_MSG(msg) // msg
#endif

char* stg_file_name;
%}

%option yylineno

NL                 [\n\r]
WS                 [\t\b ]
COMMENT            #[^\n\r]*
DIGIT              [0-9]
ALPHA              [a-zA-Z_]
ALPHANUM           [a-zA-Z_0-9]
INUMBER            {DIGIT}{DIGIT}*
RNUMBER1           {DIGIT}*\.{DIGIT}+
RNUMBER2           {DIGIT}+\.{DIGIT}*
RNUMBER            {RNUMBER1}|{RNUMBER2}
STRING             \"[^"]*\"
ID                 {ALPHA}+({DIGIT}*{ALPHA}*)*

KEY_MODEL          \.model
KEY_NAME           \.name
KEY_INPUTS         \.inputs
KEY_OUTPUTS        \.outputs
KEY_INTERNAL       \.internal
KEY_CHANNELS       \.channels
KEY_EXT_CHANNELS   \.external_channels
KEY_INT_CHANNELS   \.internal_channels
KEY_DUMMY          \.dummy
KEY_CAPACITY       \.capacity
KEY_MARKING        \.marking
KEY_CONCURRENT     \.concurrent
KEY_ORDERED        \.ordered
KEY_LATE           \.late
KEY_TIME           \.time
KEY_INITIAL_STATE  \.initial[ \t]+state
KEY_GRAPH          \.graph
KEY_STATE_GRAPH    \.state[ \t]+graph
KEY_IMPLEMENT      \.implement
KEY_WITH           with
KEY_WITHOUT        without
KEY_SLOW           \.slow
KEY_SLOWENV        \.slowenv
KEY_LABEL          \.label
KEY_END            \.end
KEY_CONC_RED       \<\|
KEY_SIMUL          \<\>

%%

{WS}+              {
//  DEBUG_MSG("WS\n");
  /* eat white spaces */
}

{COMMENT}         {
//  DEBUG_MSG("COMMENT\n");
  /* eat comments */
}

{KEY_MODEL}       {
  DEBUG_MSG("KEY_MODEL\n");
  return STG_MODEL;
}

{KEY_NAME}        {
  DEBUG_MSG("KEY_NAME\n");
  return STG_NAME;
}

{KEY_INPUTS}      {
  DEBUG_MSG("KEY_INPUTS\n");
  return STG_INPUTS;
}

{KEY_OUTPUTS}     {
  DEBUG_MSG("KEY_OUTPUTS\n");
  return STG_OUTPUTS;
}

{KEY_INTERNAL}    {
  DEBUG_MSG("KEY_INTERNAL\n");
  return STG_INTERNAL;
}

{KEY_CHANNELS}    {
  DEBUG_MSG("KEY_CHANNELS\n");
  return STG_CHANNELS;
}

{KEY_EXT_CHANNELS} {
  DEBUG_MSG("KEY_EXT_CHANNELS\n");
  return STG_EXT_CHANNELS;
}

{KEY_INT_CHANNELS} {
  DEBUG_MSG("KEY_INT_CHANNELS\n");
  return STG_INT_CHANNELS;
}

{KEY_DUMMY}       {
  DEBUG_MSG("KEY_DUMMY\n");
  return STG_DUMMY;
}

{KEY_CAPACITY}    {
  DEBUG_MSG("KEY_CAPACITY\n");
  return STG_CAPACITY;
}

{KEY_MARKING}     {
  DEBUG_MSG("KEY_MARKING\n");
  return STG_MARKING;
}

{KEY_CONCURRENT}  {
  DEBUG_MSG("KEY_CONCURRENT\n");
  return STG_CONCURRENT;
}

{KEY_ORDERED}     {
  DEBUG_MSG("KEY_ORDERED\n");
  return STG_ORDERED;
}

{KEY_LATE}        {
  DEBUG_MSG("KEY_LATE\n");
  return STG_LATE;
}

{KEY_TIME}        {
  DEBUG_MSG("KEY_TIME\n");
  return STG_TIME;
}

{KEY_INITIAL_STATE} {
  DEBUG_MSG("KEY_INITIAL_STATE\n");
  return STG_INITIAL_STATE;
}

{KEY_GRAPH}       {
  DEBUG_MSG("KEY_GRAPH\n");
  return STG_GRAPH;
}

{KEY_STATE_GRAPH} {
  DEBUG_MSG("KEY_STATE_GRAPH\n");
  return STG_STATE_GRAPH;
}

{KEY_IMPLEMENT}   {
  DEBUG_MSG("KEY_IMPLEMENT\n");
  return STG_IMPLEMENT;
}

{KEY_WITH}        {
  DEBUG_MSG("KEY_WITH\n");
  return STG_WITH;
}

{KEY_WITHOUT}     {
  DEBUG_MSG("KEY_WITHOUT\n");
  return STG_WITHOUT;
}

{KEY_SLOW}        {
  DEBUG_MSG("KEY_SLOW\n");
  return STG_SLOW;
}

{KEY_SLOWENV}     {
  DEBUG_MSG("KEY_SLOWENV\n");
  return STG_SLOWENV;
}

{KEY_LABEL}       {
  DEBUG_MSG("KEY_LABEL\n");
  return STG_LABEL;
}

{KEY_END}         {
  DEBUG_MSG("KEY_END\n");
  return STG_END;
}

{KEY_CONC_RED}    {
  DEBUG_MSG("KEY_CONC_RED\n");
  return (STG_CONC_RED);
}

{KEY_SIMUL}       {
  DEBUG_MSG("KEY_SIMUL\n");
  return (STG_SIMUL);
}

{INUMBER}          {
  DEBUG_MSG("INUMBER: "); DEBUG_MSG(yytext); DEBUG_MSG("\n");
  stg_lval.inumber = atoi(yytext);
  return STG_INUMBER;
}

{RNUMBER}          {
  DEBUG_MSG("RNUMBER: "); DEBUG_MSG(yytext); DEBUG_MSG("\n");
  stg_lval.rnumber = atof(yytext);
  return STG_RNUMBER;
}

{STRING}              {
  DEBUG_MSG("STRING: "); DEBUG_MSG(yytext); DEBUG_MSG("\n");
  stg_lval.string = namecopy(yytext);
  return STG_STRING;
}

{ID}               {
  DEBUG_MSG("ID: "); DEBUG_MSG(yytext); DEBUG_MSG("\n");
  stg_lval.string = namecopy(yytext);
  return STG_ID;
}

{NL}+             {
  DEBUG_MSG("NL\n");
  return STG_NL;
}

.                  {
  DEBUG_MSG(yytext); DEBUG_MSG("\n");
  return(yytext[0]);
}

<<EOF>>            {
  DEBUG_MSG("<EOF>\n");
  yyterminate();
  return 0;
}

%%

int yywrap()
{
  stg_lineno = 0;
  // Free the buffer which remains un-freed due to a flex bug
  yy_delete_buffer(yy_current_buffer);
  yy_init = 1;
  return 1;
}

void stg_error(char* message)
{
  Error("%s in line %d in file <%s>", message, stg_lineno, stg_file_name);
}
