#include <string.h>
#include "stg_node.h"
#include "stg_transition.h"
#include "darray.h"
#include "str_utils.h"
#include "messages.h"

const char* PATTERN_DUMMY = "%s";
const char* PATTERN_TRANSITION = "%s%c";
const char* PATTERN_TRANSITION_INDEX = "%s/%d";

const char SYMBOL_POLARITY_RISING  = '+';
const char SYMBOL_POLARITY_FALLING = '-';
const char SYMBOL_INDEX = '/';

//---------------------------------------------------------------------------
Transition::Transition(char* name, SignalArray* signals) : Node(name)
{
  this->signals = signals;
  parseName();
  this->preset = new PlaceArray(1);
  this->postset =  new PlaceArray(1);
  this->readset =  new PlaceArray(1);
}

//---------------------------------------------------------------------------
Transition::Transition(Transition& transition) : Node(transition)
{
  this->signals = transition.signals;
  parseName();
  this->preset = new PlaceArray(*transition.preset);
  this->postset =  new PlaceArray(*transition.postset);
  this->readset =  new PlaceArray(*transition.readset);
}

//---------------------------------------------------------------------------
Transition::~Transition()
{
  delete preset;
  delete postset;
  delete readset;
}

//---------------------------------------------------------------------------
void Transition::setName(char* name)
{
  Node::setName(name);
  parseName();
}

//---------------------------------------------------------------------------
void Transition::genName()
{
  delete [] name;
  name = NULL;
  char* name_new = NULL;
  
  if (signal)
  {
    switch (polarity)
    {
      case POLARITY_UNDEFINED:
        name_new = name2pattern(PATTERN_DUMMY, signal->getName());
        break;
      case POLARITY_RISING:
        name_new = name2pattern(PATTERN_TRANSITION, signal->getName(), SYMBOL_POLARITY_RISING);
        break;
      case POLARITY_FALLING:
        name_new = name2pattern(PATTERN_TRANSITION, signal->getName(), SYMBOL_POLARITY_FALLING);
        break;
    }
    if (index >= 0)
    {
      name = name2pattern(PATTERN_TRANSITION_INDEX, name_new, index);
      delete [] name_new;
    }
    else
      name = name_new;
  }
  else
    Error("Transition::genName() - signal is undefined");
}

//---------------------------------------------------------------------------
void Transition::parseName()
{
  signal = NULL;
  polarity = POLARITY_UNDEFINED; 
  index = -1;
  if (name)
  {
    char* name_cpy = namecopy(name);
    char* polarity_str = strchr(name_cpy, SYMBOL_POLARITY_RISING);
    if (polarity_str)
      polarity = POLARITY_RISING;
    else
    {    
      polarity_str = strchr(name_cpy, SYMBOL_POLARITY_FALLING);
      if (polarity_str)
        polarity = POLARITY_FALLING;
    }
    if (polarity_str)
    {
      char* index_str = strchr(polarity_str, SYMBOL_INDEX);
      if (index_str)
        index = atoi(&index_str[1]);
      polarity_str[0] = '\0';
    }
    if (signals)
    {
      for (int i = 0; i < signals->num; i++)
        if (strcmp(name_cpy, signals->arr[i]->getName()) == 0)
	{
          signal = signals->arr[i];
	  break;
	}
    }
    if (signal == NULL)
      Error("Transition::genName() - signal <%s> needed for transition <%s> is not found", name_cpy, name);
    delete [] name_cpy;
    
  }
  else
    Error("Transition::parseName() - name is undefined");
}

//---------------------------------------------------------------------------
void Transition::setSignal(Signal* signal)
{
  this->signal = signal;
  genName();
}

//---------------------------------------------------------------------------
void Transition::setPolarity(Transition::Polarity polarity)
{
  this->index = index;
  genName();
}

//---------------------------------------------------------------------------
void Transition::setIndex(int index)
{
  this->index = index;
  genName();
}
