%{
#include "stg_node.h"
#include "stg_signal.h"
#include "stg_place.h"
#include "stg_transition.h"
#include "stg_net.h"
#include "stg.h"

//#define DEBUG 1
#ifdef DEBUG
  #include <stdio.h>
  #define DEBUG_MSG(msg) fprintf(stderr, msg)
#else
  #define DEBUG_MSG(msg) // msg
#endif

STG* stg_net;

extern int stg_lex();
extern void stg_error(char*);
%}

%start stg

%union
{
  Signal* signal;
  SignalArray* signals;
  Node* node;
  NodeArray* nodes;
  char* string;
  int inumber;
  float rnumber;
}

%token <string> STG_MODEL
%token <string> STG_NAME
%token <string> STG_INPUTS
%token <string> STG_OUTPUTS
%token <string> STG_INTERNAL
%token <string> STG_CHANNELS
%token <string> STG_EXT_CHANNELS
%token <string> STG_INT_CHANNELS
%token <string> STG_DUMMY
%token <string> STG_CAPACITY
%token <string> STG_MARKING
%token <string> STG_CONCURRENT
%token <string> STG_ORDERED
%token <string> STG_LATE
%token <string> STG_TIME
%token <string> STG_INITIAL_STATE
%token <string> STG_GRAPH
%token <string> STG_STATE_GRAPH
%token <string> STG_IMPLEMENT
%token <string> STG_WITH
%token <string> STG_WITHOUT
%token <string> STG_SLOW
%token <string> STG_SLOWENV
%token <string> STG_LABEL
%token <string> STG_END
%token <string> STG_CONC_RED
%token <string> STG_SIMUL
%token <string> STG_ID
%token <inumber> STG_INUMBER
%token <rnumber> STG_RNUMBER
%token <string> STG_STRING
%token <string> STG_NL
//%type stg
%type <string> model
//%type intro
//%type intro_declaration
%type <signals> signal_def_list
%type <string> signal_def
//%type initial_state
//%type state_list
//%type state
//%type graph
//%type pn_line_list
//%type pn_line
//%type %type guard_opt
//%type guard
//%type guard_signal
%type <nodes> pn_arc_list
%type <node> node_ext
%type <node> node
%type <string> node_name
%type <string> transition
%type <inumber> instance_opt
%type <string> place_or_dummy
//%type sg_line_list
//%type sg_line
//%type sg_arc_list
//%type sg_arc
//%type outro
//%type outro_declaration
//%type marking_list
//%type marking
%type <inumber> token_num_opt
//%type implement
//%type slow_list
//%type capacity_list
//%type capacity
//%type edge_list
%type <nodes> edge
//%type time_list
//%type concurrency_reduction
//%type early_enabling
//%type trigger_events
//%type simultaneous_events
//%type fast_events
//%type slow_events
//%type concurrent_events
//%type label_list
//%type label_def
//%type label
//%type signal_list
//%type end
//%type after_end

%%

stg
  : model intro initial_state graph outro end
    {
      DEBUG_MSG("stg : model intro initial_state graph outro end\n");
      stg_net->setName($1);
    }
;

model
  :
    {
      DEBUG_MSG("model : \n");
      $$ = NULL;
    }
  | STG_NL model
    {
      DEBUG_MSG("model : STG_NL model\n");
      $$ = $2;
    }
  | STG_MODEL STG_NL
    {
      DEBUG_MSG("model : STG_MODEL STG_NL\n");
      $$ = NULL;
    }
  | STG_MODEL STG_ID STG_NL
    {
      DEBUG_MSG("model : STG_MODEL STG_ID STG_NL\n");
      $$ = $2;
    }
;

intro
  :
    {
      DEBUG_MSG("intro : \n");
    }
  | STG_NL intro
    {
      DEBUG_MSG("intro : STG_NL intro\n");
    }
  | intro intro_declaration
    {
      DEBUG_MSG("intro : intro intro_declaration\n");
    }
;

intro_declaration
  : intro_declaration STG_NL
    {
      DEBUG_MSG("intro_declaration : intro_declaration STG_NL\n");
    }
  | STG_INPUTS signal_def_list STG_NL
    {
      DEBUG_MSG("intro_declaration : STG_INPUTS signal_def_list STG_NL\n");
      for (int i = 0; i < $2->num; i++)
      {
        $2->arr[i]->setType(Signal::TYPE_INPUT);
      }
      stg_net->addSignals($2);
      delete $2;
    }
  | STG_OUTPUTS signal_def_list STG_NL
    {
      DEBUG_MSG("intro_declaration : STG_OUTPUTS signal_def_list STG_NL\n");
      for (int i = 0; i < $2->num; i++)
      {
        $2->arr[i]->setType(Signal::TYPE_OUTPUT);
      }
      stg_net->addSignals($2);
      delete $2;
    }
  | STG_INTERNAL signal_def_list STG_NL
    {
      DEBUG_MSG("intro_declaration : STG_INTERNAL signal_def_list STG_NL\n");
      for (int i = 0; i < $2->num; i++)
      {
        $2->arr[i]->setType(Signal::TYPE_INTERNAL);
      }
      stg_net->addSignals($2);
      delete $2;
    }
  | STG_DUMMY signal_def_list STG_NL
    {
      DEBUG_MSG("intro_declaration : STG_DUMMY signal_def_list STG_NL\n");
      for (int i = 0; i < $2->num; i++)
      {
        $2->arr[i]->setType(Signal::TYPE_DUMMY);
      }
      stg_net->addSignals($2);
      delete $2;
    }
  | STG_EXT_CHANNELS signal_def_list STG_NL
    {
      DEBUG_MSG("intro_declaration : STG_EXT_CHANNELS signal_def_list STG_NL\n");
      if ($2) $2->removeAll(true);
      delete $2;
    }
  | STG_INT_CHANNELS signal_def_list STG_NL
    {
      DEBUG_MSG("intro_declaration : STG_INT_CHANNELS signal_def_list STG_NL\n");
      if ($2) $2->removeAll(true);
      delete $2;
    }
;

signal_def_list
  :
    {
      DEBUG_MSG("signal_def_list : \n");
      $$ = new SignalArray(1);
    }
  | signal_def_list signal_def
    {
      DEBUG_MSG("signal_def_list : signal_def_list signal_def\n");
      $$ = $1;
      $$->add(new Signal($2));
    }
;

signal_def
  : STG_ID
  | STG_ID '(' STG_INUMBER ')'
  | STG_ID '+'
  | STG_ID '-'
  | STG_ID '-' '-'
  | STG_ID '+' '+'
  | STG_ID '-' '+'
  | STG_ID '+' '-'
  | STG_ID '(' STG_INUMBER ')' '+'
  | STG_ID '(' STG_INUMBER ')' '-'
;

initial_state
  :
    {
      DEBUG_MSG("initial_state : \n");
    }
  | STG_INITIAL_STATE state_list STG_NL
    {
      DEBUG_MSG("initial_state : STG_INITIAL_STATE state_list STG_NL\n");
    }
;

state_list
  :
    {
      DEBUG_MSG("state_list\n");
    }
  | state_list state
    {
      DEBUG_MSG("state_list : state_list state\n");
    }
;

state
  : STG_ID
    {
      DEBUG_MSG("state : STG_ID\n");
      Signal* signal_cur = stg_net->getSignal($1);
      if (signal_cur)
        signal_cur->setState(Signal::STATE_HIGH);
      delete [] $1;
    }
  | '!' STG_ID
    {
      DEBUG_MSG("state : '!' STG_ID\n");
      Signal* signal_cur = stg_net->getSignal($2);
      if (signal_cur)
        signal_cur->setState(Signal::STATE_LOW);
      delete [] $2;
    }
;

graph
  : STG_GRAPH pn_line_list
    {
      DEBUG_MSG("graph : STG_GRAPH STG_NL pn_line_list\n");
    }
  | STG_STATE_GRAPH STG_NL sg_line_list
    {
      DEBUG_MSG("graph : STG_STATE_GRAPH STG_NL sg_line_list\n");
    }
;

pn_line_list
  :
    {
      DEBUG_MSG("pn_line_list : \n");
    }
  | pn_line_list STG_NL
    {
      DEBUG_MSG("pn_line_list : pn_line_list STG_NL\n");
    }
  | pn_line_list pn_line STG_NL
    {
      DEBUG_MSG("pn_line_list : pn_line_list pn_line STG_NL\n");
    }
;

pn_line
  : node guard_opt pn_arc_list
    {
      DEBUG_MSG("pn_line : node guard_opt pn_arc_list\n");
      for (int i = 0; i < $3->num; i++)
        stg_net->addArc($1, $3->arr[i]);
      delete $1;
      $3->removeAll(true);
      delete $3;
    }
;

guard_opt
  :
  | '?' guard
;

guard
  : guard_signal
  | guard '*' guard_signal
;

guard_signal
  : '!' STG_ID
  | STG_ID
;

pn_arc_list
  :
    {
      DEBUG_MSG("pn_arc_list : \n");
      $$ = new NodeArray(1);
    }
  | pn_arc_list node_ext
    {
      DEBUG_MSG("pn_arc_list : pn_arc_list node_ext\n");
      $$ = $1;
      $$->add($2);
    }
;

node_ext
  : node '(' STG_INUMBER ')' guard_opt
    {
      DEBUG_MSG("node_ext : node '(' STG_INUMBER ')' guard_opt\n");
      $$ = $1;
    }
  | node guard_opt
    {
      DEBUG_MSG("node_ext : node guard_opt\n");
      $$ = $1;
    }
;

node
  : node_name
    {
      DEBUG_MSG("node : STG_NAME\n");
      $$ = new Node($1);
    }
;

node_name
  : transition instance_opt
    {                                                
      DEBUG_MSG("node : transition instance_opt\n");
      if ($2 < 0)
         $$ = $1;
      else
      {
        $$ = name2pattern(PATTERN_TRANSITION_INDEX, $1, $2);
        delete [] $1;
      }
    }
  | place_or_dummy
    {
      DEBUG_MSG("node : place_or_dummy\n");
      $$ = $1;
    }                                                
;                                                    
                                                     
transition
  : STG_ID '+'
    {
      DEBUG_MSG("transition : STG_ID '+'\n");
      $$ = name2pattern(PATTERN_TRANSITION, $1, SYMBOL_POLARITY_RISING);
      delete [] $1;
    }
  | STG_ID '-'
    {
      DEBUG_MSG("transition : STG_ID '-'\n");
      $$ = name2pattern(PATTERN_TRANSITION, $1, SYMBOL_POLARITY_FALLING);
      delete [] $1;
    }
;

place_or_dummy
  : STG_ID
    {
      DEBUG_MSG("place_or_dummy : STG_ID\n");
      $$ = $1;
    }
;

instance_opt
  :
    {
      DEBUG_MSG("instance_opt : \n");                 
      $$ = -1;                                       
    }
  | '/' STG_INUMBER
    {
      DEBUG_MSG("instance_opt : '/' STG_INUMBER\n");
      $$ = $2;
    }
;

sg_line_list
  :
  | sg_line_list STG_NL
  | sg_line_list sg_line STG_NL
;

sg_line
  : STG_ID sg_arc_list
;

sg_arc_list
  :
  | sg_arc_list sg_arc
;

sg_arc
  : node guard_opt STG_ID
;

outro
  :
    {
      DEBUG_MSG("outro : \n");
    }
  | outro outro_declaration
    {
      DEBUG_MSG("outro : outro outro_declaration\n");
    }
;

outro_declaration
  : STG_MARKING '{' marking_list '}' STG_NL
    {
      DEBUG_MSG("outro_declaration : STG_MARKING '{' marking_list '}' STG_NL\n");
    }
  | implement STG_NL
  | STG_CAPACITY capacity_list STG_NL
  | STG_LABEL label_list STG_NL
  | STG_SLOW slow_list STG_NL
  | STG_SLOWENV STG_NL
  | STG_CONCURRENT '{' edge_list '}' STG_NL
  | STG_ORDERED '{' edge_list '}' STG_NL
  | STG_LATE '{' edge_list '}' STG_NL
  | STG_TIME time_list STG_NL
;

marking_list
  :
  | marking_list marking
;

marking
  : node token_num_opt
    {
      DEBUG_MSG("marking : node token_num_opt\n");
      Place* place_cur = stg_net->getPlace($1->getName());
      if (place_cur)
        place_cur->setMarking($2);
      delete $1;
    }
  | edge token_num_opt
    {
      DEBUG_MSG("marking : edge token_num_opt\n");
      if ($1->num == 2)
      {
        Place* place_cur = stg_net->getPlace($1->arr[0]->getName(), $1->arr[1]->getName());
        if (place_cur)
          place_cur->setMarking($2);
      }
      $1->removeAll(true);
      delete $1;
    }
;

token_num_opt
  :
    {
      DEBUG_MSG("token_num_opt : \n");
      $$ = 1;
    }
  | '=' STG_INUMBER
    {
      DEBUG_MSG("token_num_opt : '=' STG_INUMBER\n");
      $$ = $2;
    }
;

implement
  : STG_IMPLEMENT STG_ID STG_WITH signal_list
  | STG_IMPLEMENT STG_ID STG_WITHOUT signal_list
;

slow_list
  :
  | slow_list node
;

capacity_list
  : capacity
  | capacity_list capacity
;

capacity
  : STG_ID '=' STG_INUMBER
  | edge '=' STG_INUMBER
;

edge_list
  :
  | edge_list edge
;

edge
  : '<' node ',' node '>'
    {
      DEBUG_MSG("edge : '<' node ',' node '>'\n");
      $$ = new NodeArray(2);
      $$->add($2);
      $$->add($4);
    }
;

time_list
  :
  | time_list concurrency_reduction
  | time_list early_enabling
  | time_list simultaneous_events
  | time_list concurrent_events
;

concurrency_reduction
  : node STG_CONC_RED node
;

early_enabling
  : node '>' trigger_events
;

trigger_events
  : node
  | trigger_events '>' node
;

simultaneous_events
  : fast_events '@' slow_events
;

fast_events
  : node '=' node
  | fast_events '=' node
;

slow_events
  : node
  | slow_events ',' node
;

concurrent_events
  : node STG_SIMUL node
;

label_list
  : label_def
  | label_list label_def
;

label_def
  : node '=' label
;

label
  : STG_ID
  | STG_STRING
;

signal_list
  : STG_ID
  | signal_list STG_ID
;

end
  : STG_END after_end
    {
      DEBUG_MSG("end : STG_END after_end\n");
    }
;

after_end
  :
    {
      DEBUG_MSG("after_end : \n");
    }
  | STG_NL after_end
    {
      DEBUG_MSG("after_end : STG_NL after_end\n");
    }
;

%%

