#include "le_leffort.h"



//constructor
LEffort::LEffort()
{
	myLib = NULL;
	myCircuit = NULL;
	myModule = NULL;
	myGraph = NULL;

	traces = NULL;

	myTrace = NULL;
	myPath = NULL;

	tau_ref = 0.0;
	tau_ref_rise = 0.0;
	tau_ref_fall = 0.0;
	c_in_prac_inv = 0.0;
	c_in_theo_inv = 0.0;
}




LEffort::LEffort(const LEffort &rhs)
{
	
}



//destructor
LEffort::~LEffort()
{
	if (traces != NULL)
	{	
		for (int i=0; i<traces->num; i++)
			delete traces->arr[i];
		delete traces;
		traces = NULL;
	}

	if (myCircuit != NULL)
	{
		delete myCircuit;
		myCircuit = NULL;
	}

	if (myLib != NULL)
	{
		delete myLib;
		myLib = NULL;
	}

	if (myGraph != NULL)
	{
		delete myGraph;
		myGraph = NULL;
	}
}





void LEffort::readVerilog(char *libName, char *circuitName)
{
	myLib = new Lib(NULL);
	myLib->readModules(libName);
	myLib->calcHierarchy();

	myCircuit = new Circuit(TAG_UNDEFINED, NULL, NULL, myLib);
	myCircuit->readModules(circuitName);
	myCircuit->calcHierarchy();

	myCircuit->writeModules("-");	

	myModule = myCircuit->calcModuleTop();
}



void LEffort::readSTG(char *stgName)
{
	myGraph = new Graph;
	myGraph->readASTG(stgName);
	//myGraph->writeASTG("jjj");
}




void LEffort::readParameters(char *fileName, char *fileName_slope)
{
	read_parameters(fileName, myLib);
	setParameters();

	read_slope_parameters(fileName_slope, myLib);
}






void LEffort::extractTraces()
{
	traces = new darray<Trace*>();
	myGraph->extractTraces(traces);
	setConnectionTraces();
	extractPaths();
	

// 	cout << endl << "traces: " << endl;
// 	for (int i=0; i<traces->num; i++)
// 		cout << *(traces->arr[i]) << endl;
}





void LEffort::calculateDelay()
{
	Element *element = NULL;
	Element *nextElement = NULL;
	Connection *connectionSucc = NULL;
	bool isRising = true;
	double delay_theo = 0.0;
	double delay_prac = 0.0;
	double delay_slope_theo = 0.0;
	double delay_slope_prac = 0.0;


	for (int i=0; i<traces->num; i++)
		if (traces->arr[i]->getPaths() != NULL)
		{
			if (traces->arr[i]->getTransitions()->arr[0]->getPolarity() == Transition::POLARITY_RISING) 
				isRising = true;
			else
				isRising = false;

			for (int j=0; j<traces->arr[i]->getPaths()->num; j++)
			{
				delay_theo = 0.0;
				delay_prac = 0.0;
				delay_slope_theo = 0.0;
				delay_slope_prac = 0.0;

				for (int k=0; k<traces->arr[i]->getPaths()->arr[j]->getPath()->num-1; k++)
				{
					element = traces->arr[i]->getPaths()->arr[j]->getPath()->arr[k];
					nextElement = traces->arr[i]->getPaths()->arr[j]->getPath()->arr[k+1];

					if (k == 0)	 //set initial input slope
					{
						element->setSlopeTheo(SLOPE_INITIAL);
						element->setSlopePrac(SLOPE_INITIAL);
					}

					//change edge only if gate is not a buffer
					if (strncmp(element->getInstance()->getModuleName(), BUFFER, strlen(BUFFER)) != 0)
						(isRising) ? isRising = false: isRising = true;

					setDelay(element, nextElement, isRising);
					
					delay_theo = element->getDelayTheo() + delay_theo;
					delay_prac = element->getDelayPrac() + delay_prac;
					delay_slope_theo = element->getDelaySlopeTheo() + delay_slope_theo;
					delay_slope_prac = element->getDelaySlopePrac() + delay_slope_prac;
				}

				traces->arr[i]->getPaths()->arr[j]->setDelayTheo(delay_theo);	
				traces->arr[i]->getPaths()->arr[j]->setDelayPrac(delay_prac);
				traces->arr[i]->getPaths()->arr[j]->setDelaySlopeTheo(delay_slope_theo);
				traces->arr[i]->getPaths()->arr[j]->setDelaySlopePrac(delay_slope_prac);
			}
		traces->arr[i]->getPaths()->sort(Path::compare);
		}
	traces->sort(Trace::compare);
}




void LEffort::printTraces()
{
	for (int i=0; i<traces->num; i++)
		if (traces->arr[i]->getPaths() != NULL)
		{
			printTrace(traces->arr[i]);
			for (int j=0; j<traces->arr[i]->getPaths()->num; j++)
			{
				cout << "path " << j+1 << ": ";
				printPath(traces->arr[i]->getPaths()->arr[j]);
			}
			cout << endl;
		}
}		





void LEffort::printTraces(char *fileName)
{
	ofstream fout(fileName);

	for (int i=0; i<traces->num; i++)
		if (traces->arr[i]->getPaths() != NULL)
		{
			for (int j=0; j<traces->arr[i]->getPaths()->num; j++)
			{
				printTrace(traces->arr[i], fout);

				fout << "	path " << j+1 << ": ";
				printPath(traces->arr[i]->getPaths()->arr[j], fout);
				fout << endl;
			}
			
		}

	fout.close();
}		





//####################################################################################
//overload operators
//####################################################################################

ostream& operator << (ostream &os, const LEffort &le)
{
	
	if (le.getTraces() != NULL)
	{
		cout << endl << "traces: " << endl;
		for (int i=0; i<le.getTraces()->num; i++)
			cout << *(le.getTraces()->arr[i]) << endl;
	}

	return os;
}





///////////////////////////////////////////////////////////////////////////////////////
//	private methods
///////////////////////////////////////////////////////////////////////////////////////




//
//	set traces
//



void LEffort::setConnectionTraces()
{
	for (int i=0; i<traces->num; i++)
		for (int j=0; j<traces->arr[i]->getTransitions()->num; j++)
			if ((traces->arr[i]->getTransitions()->arr[j]->getSignal()->getType() == Signal::TYPE_OUTPUT) || (traces->arr[i]->getTransitions()->arr[j]->getSignal()->getType() == Signal::TYPE_INPUT))
		 		traces->arr[i]->set(findConnection(traces->arr[i]->getTransitions()->arr[j]->getSignal()->getName()));
}



Connection * LEffort::findConnection(char *name)
{
	for (int i=0; i<myModule->instances->num; i++)
		for (int j=0; j<myModule->instances->arr[i]->connections->num; j++)
			for (int k=0; k<myModule->instances->arr[i]->connections->arr[j]->nets->num; k++)
				if (strcmp(name, myModule->instances->arr[i]->connections->arr[j]->nets->arr[k]->getName()) == 0)
					return myModule->instances->arr[i]->connections->arr[j];
		
	return NULL;
}





void LEffort::extractPaths()
{
	resetTags();
	for (int i=0; i<traces->num; i++)
	{ 
		myTrace = traces->arr[i];
		myPath = new Path;

		if (myTrace->getConnections()->num >= 2)
			extractPath(myTrace->getConnections()->arr[0], myTrace->getConnections()->arr[1], 2);
		
		delete myPath;
	}
}




void LEffort::extractPath(Connection *currentConnection, Connection *nextConnection, int pos)
{
	for (int i=0; i<currentConnection->nets->num; i++)
		if (!currentConnection->nets->arr[i]->testTag(TAG_PROCESSED))
		{
			if (currentConnection->nets->arr[i]->testTag(TAG_PORT_OUTPUT))
			{	
				if (isNameEqual(currentConnection->nets->arr[i], nextConnection)) 
				{	
					if (isTracesEnd(currentConnection->nets->arr[i]))	//set path
					{
						Element *element = new Element(currentConnection, NULL);
						myPath->set(element);
						myTrace->set(myPath);
						myPath->getPath()->remove(element, 1);
					
						return;
					}
		
					nextConnection = myTrace->getConnections()->arr[pos]; //set next output connection
					pos++;
					
					for (int j=0; j<currentConnection->nets->arr[i]->postset->num; j++)
					{
						Element *element = new Element(currentConnection, currentConnection->nets->arr[i]->postset->arr[j]);
						myPath->set(element);
						currentConnection->nets->arr[i]->orTag(TAG_PROCESSED);
						extractPath(getInstancesOutput(element->getInstance()), nextConnection, pos);
						currentConnection->nets->arr[i]->andTag(~TAG_PROCESSED);
						myPath->getPath()->remove(element, 1);
					}	
				}
			}
			else //current net is non-output, go to next instance
			{
				if (currentConnection->nets->arr[i]->postset == NULL)
					return;
				
				for (int j=0; j<currentConnection->nets->arr[i]->postset->num; j++)
				{	
					Element *element = new Element(currentConnection, currentConnection->nets->arr[i]->postset->arr[j]);
					myPath->set(element);
					currentConnection->nets->arr[i]->orTag(TAG_PROCESSED);
					extractPath(getInstancesOutput(element->getInstance()), nextConnection, pos);
					currentConnection->nets->arr[i]->andTag(~TAG_PROCESSED);
					myPath->getPath()->remove(element, 1);
				
				}	
			}
		}
}




bool LEffort::isNameEqual(Net *net, Connection *connection)
{
	for (int i=0; i<connection->nets->num; i++)
		if (strcmp(net->getName(), connection->nets->arr[i]->getName()) == 0)
			return true;

	return false;
}




bool LEffort::isTracesEnd(Net *myNet)
{
	Connection *terminal = myTrace->getConnections()->arr[myTrace->getConnections()->num-1];
	for (int i=0; i<terminal->nets->num; i++)
		if (strcmp(myNet->getName(), terminal->nets->arr[i]->getName()) == 0)
			return true;
	return false;
}




Connection * LEffort::getInstancesOutput(Instance *instance)
{
	for (int i=instance->connections->num-1; i>=0; i--)
		if (instance->connections->arr[i]->testTag(TAG_PORT_OUTPUT))
				return instance->connections->arr[i];
}




void LEffort::resetTags()
{
	for (int i=0; i<myModule->instances->num; i++)
		for (int j=0; j<myModule->instances->arr[i]->connections->num; j++)
			for (int k=0; k<myModule->instances->arr[i]->connections->arr[j]->nets->num; k++)
				myModule->instances->arr[i]->connections->arr[j]->nets->arr[k]->andTag(~TAG_PROCESSED);
}




//
//	set parameters
//





void LEffort::setParameters()
{
	setRefParameters();

	for (int i=0; i<myLib->modules->num; i++)
		for (int j=0; j<myLib->modules->arr[i]->ports->num; j++)
			if (myLib->modules->arr[i]->ports->arr[j]->testTag(TAG_PORT_INPUT))
				setParameters(myLib->modules->arr[i], myLib->modules->arr[i]->ports->arr[j]->getName());
				//for (int k=0; k<myLib->modules->arr[i]->ports->arr[j]->nets->num; k++)
					//setParameters(myLib->modules->arr[i], myLib->modules->arr[i]->ports->arr[j]->nets->arr[k]->getName());
}




void LEffort::setRefParameters()
{
	Module *refInverter = myLib->getModule((char*)REF_INVERTER);

	double d1_rise = getParameter(refInverter, (char*)REF_INVERTER_PIN, (char*)D1_PRAC_RISE);
	double d2_rise = getParameter(refInverter, (char*)REF_INVERTER_PIN, (char*)D2_PRAC_RISE);
	double d1_fall = getParameter(refInverter, (char*)REF_INVERTER_PIN, (char*)D1_PRAC_FALL);
	double d2_fall = getParameter(refInverter, (char*)REF_INVERTER_PIN, (char*)D2_PRAC_FALL);
	double c_in_prac = getParameter(refInverter, (char*)REF_INVERTER_PIN, (char*)C_PRAC_IN);

	tau_ref_rise = (d1_rise - d2_rise) * c_in_prac / (C1_PRAC_OUT - C2_PRAC_OUT);
	tau_ref_fall = (d1_fall - d2_fall) * c_in_prac / (C1_PRAC_OUT - C2_PRAC_OUT);
	tau_ref = (tau_ref_rise + tau_ref_fall ) / 2;

	
	c_in_prac_inv = getParameter(refInverter, (char*)REF_INVERTER_PIN, (char*)C_PRAC_IN);
	c_in_theo_inv = getParameter(refInverter, (char*)REF_INVERTER_PIN, (char*)C_THEO_IN);

	cout << "ref inverter (tau, tau_rise, tau_fall, c_in_prac, c_in_theo): " << tau_ref << ", " << tau_ref_rise << ", " << tau_ref_fall << ", "<< c_in_prac_inv << ", " << c_in_theo_inv << endl;

}




void LEffort::setParameters(Module *module, char *pinName)
{
	if (module->parameters == NULL)
		return;

	double d1_rise = getParameter(module, pinName, (char*)D1_PRAC_RISE);
	double d2_rise = getParameter(module, pinName, (char*)D2_PRAC_RISE); 
	double d1_fall = getParameter(module, pinName, (char*)D1_PRAC_FALL); 
	double d2_fall = getParameter(module, pinName, (char*)D2_PRAC_FALL); 
	double c_in_prac = getParameter(module, pinName, (char*)C_PRAC_IN);
	double c_in_theo = getParameter(module, pinName, (char*)C_THEO_IN); 
	double g = getParameter(module, pinName, (char*)G_THEO);

	// tau*g = [(d1-d2)*Cin]/(Cout1-Cout2)
	double g_rise = (d1_rise - d2_rise) * c_in_prac / (C1_PRAC_OUT - C2_PRAC_OUT) / tau_ref;
	double g_fall = (d1_fall - d2_fall) * c_in_prac / (C1_PRAC_OUT - C2_PRAC_OUT) / tau_ref;

	
	//if (c_in_theo == 0) //use practical value if theoretical is zero
	//{
	//	c_in_theo = c_in_prac / c_in_prac_inv * c_in_theo_inv;
	//	setParameter(module, pinName, (char*)C_THEO_IN, c_in_theo); 
	//}
	//double g = c_in_theo / c_in_theo_inv;

	// tau*p = d1-tau*g*h1
	double p_rise =  (d1_rise - (g_rise * tau_ref * C1_PRAC_OUT / c_in_prac)) / tau_ref;
	double p_fall =  (d1_fall - (g_fall * tau_ref * C1_PRAC_OUT / c_in_prac)) / tau_ref;
	//double p = (p_rise + p_fall) / 2;
	double p = getParameter(module, (char*)P_THEO);
	
	setParameter(module, pinName, (char*)G_THEO, g);
	setParameter(module, pinName, (char*)G_PRAC_RISE, g_rise);
	setParameter(module, pinName, (char*)G_PRAC_FALL, g_fall);

	setParameter(module, pinName, (char*)P_PRAC_RISE, p_rise);
	setParameter(module, pinName, (char*)P_PRAC_FALL, p_fall);
	setParameter(module, pinName, (char*)P_THEO, p);
}





void LEffort::setParameter(Module *module, char *pinName, char *parameterName, double value)
{
	char *name = name2pattern("%s%s%s", pinName, SEPERATOR, parameterName);
	Parameter *parameter = module->getParameter(name);
	delete []name;

	if (parameter == NULL)
	{
		parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", pinName, SEPERATOR, parameterName), value);
		module->addParameter(parameter, -1);
	}
	else
		parameter->setValue(value);
}




double LEffort::getParameter(Module *module, char *parameterName)
{
	double value = 0.0;
	char *name = name2pattern("%s", parameterName);
	Parameter *parameter = module->getParameter(name);
	delete []name;

	if (parameter != NULL)
		value = parameter->getValue();

	return value;
}



double LEffort::getParameter(Module *module, char *pinName, char *parameterName)
{
	double value = 0.0;
	char *name = name2pattern("%s%s%s", pinName, SEPERATOR, parameterName);
	Parameter *parameter = module->getParameter(name);
	delete []name;

	if (parameter != NULL)
		value = parameter->getValue();

	return value;
}



//
//	calculate delay
//




void LEffort::setDelay(Element *element, Element *nextElement, bool isRising)
{
	Instance *instance = element->getInstance();
	Connection *connection = getInstancesInputConnection(instance, element->getConnection()); //connection corresponding to instances input
	Module *module = myLib->getModule(instance->getModuleName());

	Connection *connectionSucc = nextElement->getConnection();
	

	bool isTheoretic = true;

	//set theo delay 
	element->setGtheo(getLogicalEffort(isTheoretic, isRising, module, connection->getName()));
	element->setHtheo(getElectricalEffort(isTheoretic, module, connection->getName(), connectionSucc->nets->arr[0]));
	element->setPtheo(getParasiticDelay(isTheoretic, isRising, module, connection->getName()));
	if(tau_ref > 0)
	{
		element->setDelayTheo(tau_ref *(element->getGtheo()*element->getHtheo() + element->getPtheo()));
	}
	else
	{
		element->setDelayTheo(element->getGtheo()*element->getHtheo() + element->getPtheo());
	}

	//set theo delay concidering slope
	double load = calculateCout(isTheoretic, connectionSucc->nets->arr[0]);
	element->setDelaySlopeTheo(getDelaySlope(isTheoretic, isRising, element->getSlopeTheo(), load, module, connection->getName()));
	//set output slope of current element as input slope to next one
	nextElement->setSlopeTheo(getOutputSlope(isTheoretic, isRising, element->getSlopeTheo(), load, module, connection->getName()));


	isTheoretic = false;

	//set prac delay
	element->setGprac(getLogicalEffort(isTheoretic, isRising, module, connection->getName()));
	element->setHprac(getElectricalEffort(isTheoretic, module, connection->getName(), connectionSucc->nets->arr[0]));
	element->setPprac(getParasiticDelay(isTheoretic, isRising, module, connection->getName()));
	if (isRising)
		element->setDelayPrac(tau_ref_rise *(element->getGprac()*element->getHprac() + element->getPprac()));
	else
		element->setDelayPrac(tau_ref_fall *(element->getGprac()*element->getHprac() + element->getPprac()));

	//set prac delay concidering slope
	load = calculateCout(isTheoretic, connectionSucc->nets->arr[0]);
	element->setDelaySlopePrac(getDelaySlope(isTheoretic, isRising, element->getSlopePrac(), load, module, connection->getName()));
	//set output slope of current element as input slope to next one
	nextElement->setSlopePrac(getOutputSlope(isTheoretic, isRising, element->getSlopePrac(), load, module, connection->getName()));

	
}





double LEffort::getLogicalEffort(bool isTheoretic, bool isRising, Module *module, char *pinName)
{
	double effort = 0.0;

	if (isTheoretic)
		effort = getParameter(module, pinName, (char*)G_THEO);
	else
	{
		if (isRising)
			effort = getParameter(module, pinName, (char*)G_PRAC_RISE);
		else
			effort = getParameter(module, pinName, (char*)G_PRAC_FALL);
	}
	//cout << "g: " << effort << endl;

	return effort;
}






double LEffort::getElectricalEffort(bool isTheoretic, Module *module, char *pinName, Net *net)
{
	double c_in = 0.0;
	double c_out = 0.0;

	c_out = calculateCout(isTheoretic, net);
	if (isTheoretic)
		c_in = getParameter(module, pinName, (char*)C_THEO_IN);	
	else
		c_in = getParameter(module, pinName, (char*)C_PRAC_IN);

	//cout << "h: " <<pinName << " "  << c_in << ", " << c_out << endl; 

	return (c_out / c_in);
}




double LEffort::getParasiticDelay(bool isTheoretic, bool isRising, Module *module, char *pinName)
{
	double parasitic = 0.0;

	if (isTheoretic)
		parasitic = getParameter(module, pinName, (char*)P_THEO);
	else
	{
		if (isRising)
			parasitic = getParameter(module, pinName, (char*)P_PRAC_RISE);
		else
			parasitic = getParameter(module, pinName, (char*)P_PRAC_FALL);
	}
	//cout << "p: " << parasitic << endl;

	return parasitic;
}





double LEffort::getOutputSlope(bool isTheoretic, bool isRising, double slope_in, double load_out, Module *module, char *pinName)
{
	double slope_out = 0.0;
	double w = 0.0;
	double x = 0.0;
	double y = 0.0;
	double z = 0.0;


	if (isTheoretic)
	{
		double y_rise = getParameter(module, pinName, (char*)Y2_RISE);
		double y_fall = getParameter(module, pinName, (char*)Y2_FALL);

		double z_rise = getParameter(module, pinName, (char*)Z2_RISE);
		double z_fall = getParameter(module, pinName, (char*)Z2_FALL);

		y = (y_rise + y_fall) / 2;
		z = (z_rise + z_fall) / 2;

		slope_out = tau_ref*load_out/c_in_theo_inv + y*slope_in + z;
	}	
	else
	{
		if (isRising)
		{
			w = getParameter(module, pinName, (char*)W2_RISE);
			x = getParameter(module, pinName, (char*)X2_RISE);
			y = getParameter(module, pinName, (char*)Y2_RISE);
			z = getParameter(module, pinName, (char*)Z2_RISE);
		}
		else
		{
			w = getParameter(module, pinName, (char*)W2_FALL);
			x = getParameter(module, pinName, (char*)X2_FALL);
			y = getParameter(module, pinName, (char*)Y2_FALL);
			z = getParameter(module, pinName, (char*)Z2_FALL);
		}

		slope_out = w*slope_in*load_out + x*load_out + y*slope_in + z;
	}

	
// 	if (isTheoretic)
// 	{
// 		cout << module->getName() << " " << pinName << endl;
// 		cout <<  "slope_out: " << slope_out << " = "  << load_out << "/" << c_in_theo_inv <<  " + " << y << "*" << slope_in << " + " << z << endl;
// 	}
// 	else
// 		cout <<  "slope_out: " << slope_out << " = " << w << "*" << slope_in << "*" << load_out << " + " << x << "*" << load_out << " + " << y << "*" << slope_in << " + " << z << endl;
// 	cout << "s: " << slope_in << " " << slope_out << endl;

	return slope_out;
}




double LEffort::getDelaySlope(bool isTheoretic, bool isRising, double slope_in, double load_out, Module *module, char *pinName)
{
	double delay_slope = 0.0;
	double w = 0.0;
	double x = 0.0;
	double y = 0.0;
	double z = 0.0;


	if (isTheoretic)
	{
		double y_rise = getParameter(module, pinName, (char*)Y1_RISE);
		double y_fall = getParameter(module, pinName, (char*)Y1_FALL);

		double z_rise = getParameter(module, pinName, (char*)Z1_RISE);
		double z_fall = getParameter(module, pinName, (char*)Z1_FALL);

		y = (y_rise + y_fall) / 2;
		z = (z_rise + z_fall) / 2;

		delay_slope = tau_ref*load_out/c_in_theo_inv + y*slope_in + z;
	}	
	else
	{
		if (isRising)
		{
			w = getParameter(module, pinName, (char*)W1_RISE);
			x = getParameter(module, pinName, (char*)X1_RISE);
			y = getParameter(module, pinName, (char*)Y1_RISE);
			z = getParameter(module, pinName, (char*)Z1_RISE);
		}
		else
		{
			w = getParameter(module, pinName, (char*)W1_FALL);
			x = getParameter(module, pinName, (char*)X1_FALL);
			y = getParameter(module, pinName, (char*)Y1_FALL);
			z = getParameter(module, pinName, (char*)Z1_FALL);
		}

		delay_slope = w*slope_in*load_out + x*load_out + y*slope_in + z;
	}

// 	if (isTheoretic)
// 		cout <<  "delay_slope: " << delay_slope << " = "  << load_out << "/" << c_in_theo_inv <<  " + " << y << "*" << slope_in << " + " << z << endl;
// 	else
// 		cout <<  "delay_slope: " << delay_slope << " = " << w << "*" << slope_in << "*" << load_out << " + " << x << "*" << load_out << " + " << y << "*" << slope_in << " + " << z << endl;
	

	//cout << "delay slope: " << delay_slope << endl;

	return delay_slope;
}




double LEffort::calculateCout(bool isTheoretic, Net *net)
{
	double c_out = 0.0;

	if (net->postset == NULL) 
	{
		if (isTheoretic) 
			c_out = c_in_theo_inv; 
		else 
			c_out = c_in_prac_inv;
	}
	else
		for (int i=0; i<net->postset->num; i++)
			c_out = (getCin(isTheoretic, net, net->postset->arr[i])) + c_out;

	return c_out;
}




//net is an input of instance
double LEffort::getCin(bool isTheoretic, Net *net, Instance *instance)
{
	double c = 0.0;

	for (int i=0; i<instance->connections->num; i++)
		for (int j=0; j<instance->connections->arr[i]->nets->num; j++)
			if (strcmp(net->getName(), instance->connections->arr[i]->nets->arr[j]->getName()) == 0)
			{
				if (isTheoretic)
					c = getParameter(myLib->getModule(instance->getModuleName()), instance->connections->arr[i]->getName(), (char*)C_THEO_IN) + c;
				else
					c = getParameter(myLib->getModule(instance->getModuleName()), instance->connections->arr[i]->getName(), (char*)C_PRAC_IN) + c;
			}	
		
	return c;
}




Connection * LEffort::getInstancesInputConnection(Instance *instance, Connection *connection)
{
	for (int i=0; i<instance->connections->num; i++)
		for (int j=0; j<instance->connections->arr[i]->nets->num; j++)
			for (int k=0; k<connection->nets->num; k++)
				if (strcmp(connection->nets->arr[k]->getName(), instance->connections->arr[i]->nets->arr[j]->getName()) == 0)
					return instance->connections->arr[i];
	
	return NULL;
}




//
//	print functions
//




void LEffort::printTrace(Trace *trace, ofstream &fout)
{
	if (trace->getTransitions() != NULL)
	{
		for (int i=0; i<trace->getTransitions()->num; i++)
		{
			Transition *transition = trace->getTransitions()->arr[i];
			bool isInOutput = true;

			if ((transition->getSignal()->getType() == Signal::TYPE_INPUT) || (transition->getSignal()->getType() == Signal::TYPE_OUTPUT))
				isInOutput = false;

			if (isInOutput) 
				fout << "(";
			fout << transition->getName();
			if (isInOutput) 
				fout << ")";
			if (i < (trace->getTransitions()->num-1))
				fout << ", ";
		}
		//fout << endl;
	}

}



void LEffort::printPath(Path *path, ofstream &fout)
{
	fout.precision(3);
	Element *element = NULL;


	fout << "	" << path->getDelayTheo() << "	" << path->getDelayPrac() << "	" << path->getDelaySlopeTheo() << "	" << path->getDelaySlopePrac() << "	";

	//output path
	for (int i=0; i<path->getPath()->num; i++)
	{
		element = path->getPath()->arr[i];

		if (element->getConnection() != NULL)
			fout << element->getConnection()->genFullName();

		if (element->getInstance() != NULL)
			fout << " -> " << element->getInstance()->getName();
			//fout << " -> " << element->getInstance()->getModuleName();

		if (i < (path->getPath()->num-1))
			fout << " -> ";
	}
	//fout << endl;

}





void LEffort::printTrace(Trace *trace)
{
	if (trace->getTransitions() != NULL)
	{
		for (int i=0; i<trace->getTransitions()->num; i++)
		{
			Transition *transition = trace->getTransitions()->arr[i];
			bool isInOutput = true;

			if ((transition->getSignal()->getType() == Signal::TYPE_INPUT) || (transition->getSignal()->getType() == Signal::TYPE_OUTPUT))
				isInOutput = false;

			if (isInOutput) 
				cout << "(";
			cout << transition->getName();
			if (isInOutput) 
				cout << ")";
			if (i < (trace->getTransitions()->num-1))
				cout << ", ";
		}
		cout << endl;
	}

}




void LEffort::printPath(Path *path)
{
	cout.precision(3);
	Element *element = NULL;

	//output path
	for (int i=0; i<path->getPath()->num; i++)
	{
		element = path->getPath()->arr[i];

		if (element->getConnection() != NULL)
			cout << element->getConnection()->genFullName();

		if (element->getInstance() != NULL)
			cout << " -> " << element->getInstance()->getName();

		if (i < (path->getPath()->num-1))
			cout << " -> ";
	}
	cout << endl;

	
	cout << path->getDelayTheo() << "	" << path->getDelayPrac() << "	" << path->getDelaySlopeTheo() << "	"<< path->getDelaySlopePrac() << endl;

	//output delay (obtained using theoretic values) corresponding to path 
	cout << " " << path->getDelayTheo() << " = ";
	for (int i=0; i<path->getPath()->num-1; i++)
	{
		element = path->getPath()->arr[i];

		cout << element->getDelayTheo();
		if (i < (path->getPath()->num-2))
			cout << " + ";
	}
	cout << " (delay_theo)" << endl;

	cout << " = ";
	for (int i=0; i<path->getPath()->num-1; i++)
	{
		element = path->getPath()->arr[i];

		cout << tau_ref << "*(" << element->getGtheo() << "*" << element->getHtheo() << " + " << element->getPtheo() << ")";
		if (i < (path->getPath()->num-2))
			cout << " + ";
	}
	cout << endl;

	
	//output delay (obtained using practical) corresponding to path 
	cout << " " << path->getDelayPrac() << " = ";
	for (int i=0; i<path->getPath()->num-1; i++)
	{
		element = path->getPath()->arr[i];

		cout << element->getDelayPrac();
		if (i < (path->getPath()->num-2))
			cout << " + ";
	}
	cout << " (delay_prac)" << endl;

// 	cout << " = ";
// 	for (int i=0; i<path->getPath()->num-1; i++)
// 	{
// 		element = path->getPath()->arr[i];
// 
// 		cout << "(" << element->getGprac() << "*" << element->getHprac() << " + " << element->getPprac() << ")";
// 		if (i < (path->getPath()->num-2))
// 			cout << " + ";
// 	}
// 	cout << endl;


	

//output delay (obtained using theo values for g and h and taking into account the slope) corresponding to path 
	cout << " " << path->getDelaySlopeTheo() << " = ";
	for (int i=0; i<path->getPath()->num-1; i++)
	{
		element = path->getPath()->arr[i];

		cout << element->getDelaySlopeTheo() << "[" << element->getSlopeTheo() << "]"; 
		if (i < (path->getPath()->num-2))
			cout << " + ";
	}
	cout << " (delay_slope_theo[slope])" << endl;
// 
// 	cout << " = ";
// 	for (int i=0; i<path->getPath()->num-1; i++)
// 	{
// 		element = path->getPath()->arr[i];
// 
// 		cout << element->getTauSlope() << "*(" << element->getGprac() << "*" << element->getHprac() << " + " << element->getPprac() << ")";
// 		if (i < (path->getPath()->num-2))
// 			cout << " + ";
// 	}
// 	cout << endl;

	//output delay (obtained using practic values and taking into account the slope) corresponding to path 
	cout << " " << path->getDelaySlopePrac() << " = ";
	for (int i=0; i<path->getPath()->num-1; i++)
	{
		element = path->getPath()->arr[i];

		cout << element->getDelaySlopePrac() << "[" << element->getSlopePrac() << "]"; 
		if (i < (path->getPath()->num-2))
			cout << " + ";
	}
	cout << " (delay_slope_prac[slope])" << endl;
// 
// 	cout << " = ";
// 	for (int i=0; i<path->getPath()->num-1; i++)
// 	{
// 		element = path->getPath()->arr[i];
// 
// 		cout << element->getTauSlope() << "*(" << element->getGprac() << "*" << element->getHprac() << " + " << element->getPprac() << ")";
// 		if (i < (path->getPath()->num-2))
// 			cout << " + ";
// 	}
// 	cout << endl;

}



