#ifndef LEFFORT_H
#define LEFFORT_H


#include <fstream.h>
#include "sv_lib.h"
#include "sv_circuit.h"
#include "le_graph.h"
#include "le_trace.h"


//////////////////////////////////////////
//parameters

#define SEPERATOR       	"_"
#define REF_INVERTER    	"IN1"
#define REF_INVERTER_PIN 	"A"

//input LE parameter
#define D1_PRAC_RISE      "d1_rise"
#define D2_PRAC_RISE      "d2_rise"
#define D1_PRAC_FALL      "d1_fall"
#define D2_PRAC_FALL      "d2_fall"

#define C_PRAC_IN      		"c_in_prac"
#define C_THEO_IN      		"c_in_theo"

//logical efforts
#define G_PRAC_RISE    		"g_rise"
#define G_PRAC_FALL    		"g_fall"
#define G_THEO    				"g"

//parasitics
#define P_PRAC_RISE    		"p_rise"
#define P_PRAC_FALL    		"p_fall"
#define P_THEO    				"p"

//parameter slope

#define W1_RISE      "w1_rise"
#define W2_RISE      "w2_rise"
#define W1_FALL      "w1_fall"
#define W2_FALL      "w2_fall"

#define X1_RISE      "x1_rise"
#define X2_RISE      "x2_rise"
#define X1_FALL      "x1_fall"
#define X2_FALL      "x2_fall"

#define Y1_RISE      "y1_rise"
#define Y2_RISE      "y2_rise"
#define Y1_FALL      "y1_fall"
#define Y2_FALL      "y2_fall"

#define Z1_RISE      "z1_rise"
#define Z2_RISE      "z2_rise"
#define Z1_FALL      "z1_fall"
#define Z2_FALL      "z2_fall"

//output capacitance corresponding to logical effort parameters
const double C1_PRAC_OUT = 0.015;		//0.015pF
const double C2_PRAC_OUT = 0.15;	 	//0.15pF

//initial slope
const double SLOPE_INITIAL = 0.3;

#define BUFFER      "BU"


extern void read_parameters(char *, Lib *);
extern void read_slope_parameters(char *, Lib *);



class LEffort
{
	public:
		//constructor & destructor
		LEffort();
		LEffort(const LEffort &);
		~LEffort();

		
		//	public accessor methods
		//==========================
		
		darray<Trace*> * getTraces() const { return traces; }
		
    //	public methods
    //==================

		void readVerilog(char *, char *);	
		void readParameters(char *, char *);
		void readSTG(char *);
		void readLEparameters(char *);


		void extractTraces();	
		void calculateDelay();

		void printTraces();
		void printTraces(char *);


    //overload operators
		friend ostream& operator << (ostream &, const LEffort &);


		//	public member variables
		//===========================
		

		

	private:

		//	private methods
		//===================

		//set traces
		void setConnectionTraces();
		Connection * findConnection(char *);
		void extractPaths();
		void extractPath(Connection *, Connection *, int);
		bool isNameEqual(Net *, Connection *);
		bool isTracesEnd(Net *);
		Connection * getInstancesOutput(Instance *);
		void resetTags();

		//set parammeter
		void setParameters();
		void setRefParameters();
		void setParameters(Module *, char *);
		void setParameter(Module *, char *, char *, double);
		double getParameter(Module *, char *, char *);
		double getParameter(Module *, char *);

		//calculate delay
		void setDelay(Element *, Element *, bool);
		double getLogicalEffort(bool, bool, Module *, char *);
		double getElectricalEffort(bool, Module *, char *, Net *);
		double getParasiticDelay(bool, bool, Module *, char *);
		double getOutputSlope(bool, bool, double, double, Module *, char *);
		double getDelaySlope(bool, bool, double, double, Module *, char *);
		double calculateCout(bool, Net *);
		double getCin(bool, Net *, Instance *);
		Connection * getInstancesInputConnection(Instance *, Connection *);


		//print traces
		void printTrace(Trace *);
		void printTrace(Trace *,ofstream &);
		void printPath(Path *);
		void printPath(Path *,ofstream &);


    //	private member variables
		//===========================

		Lib *myLib;
		Circuit *myCircuit;
		Module *myModule;
		Graph *myGraph;

		darray<Trace*> * traces;
		darray<Path*> *paths;

		Trace *myTrace;
		Path *myPath;

		double tau_ref; //tau_rise of reference inverter (tau_ref_rise + tau_ref_fall) / 2
		double tau_ref_rise;
		double tau_ref_fall;
		double c_in_prac_inv;
		double c_in_theo_inv;
    
};

#endif

