%{
#include <stdio.h>
#include<stdlib.h>
#include "str_utils.h"
#include "sv_lib.h"
#include "le_leffort.h"


extern FILE *parameter_slope_in;
extern int yyparse();
extern int yylex();
void parameter_slope_error(char *);


Lib *lib_slope = NULL;
Module *module_slope = NULL;

%}


%start source

%union
 {
	char * name;
	double value;
 }

%token <name> YYBEGIN
%token <name> YYEND
%token <value> YYNUM
%token <name> YYID




%%

source
: read_parameters
{
	printf("\nparameter reading is complete\n");
}
;


read_parameters
: parameter_list
;

parameter_list
: module_parameter
| module_parameter parameter_list
;

module_parameter
: YYBEGIN '['module_name']' pin_parameter_list YYEND;

module_name
:YYID
{
	module_slope = lib_slope->getModule((char*)$1);
	//printf("\n\n Module name: %s\n ", module_slope->getName());
	if (module_slope == NULL)
	{
		printf("\n Warning: No module in Library!");
		module_slope = new Module;
		module_slope->setName((char*)$1);
	}
	else
		delete []$1;
};


pin_parameter_list
: pin_parameter_delay 
| pin_parameter_slope
| pin_parameter_delay pin_parameter_list
| pin_parameter_slope pin_parameter_list
;



pin_parameter_delay
: YYID YYNUM YYNUM YYNUM YYNUM YYNUM YYNUM YYNUM YYNUM YYNUM
{
	Parameter *parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, W1_RISE), $2);
	module_slope->addParameter(parameter, -1);
	//printf("w1 rise: %f ",$2);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, X1_RISE), $3);
	module_slope->addParameter(parameter, -1);
	//printf("x1 rise: %f ",$3);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, Y1_RISE), $4);
	module_slope->addParameter(parameter, -1);
	//printf("y1 rise: %f ",$4);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, Z1_RISE), $5);
	module_slope->addParameter(parameter, -1);
	//printf("z1 rise: %f ",$5);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, W1_FALL), $6);
	module_slope->addParameter(parameter, -1);
	//printf("w1 fall: %f ",$6);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, X1_FALL), $7);
	module_slope->addParameter(parameter, -1);
	//printf("x1 fall: %f ",$7);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, Y1_FALL), $8);
	module_slope->addParameter(parameter, -1);
	//printf("y1 fall: %f ",$8);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, Z1_FALL), $9);
	module_slope->addParameter(parameter, -1);
	//printf("z1 fall: %f ",$9);
	
	//parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, C_PRAC_IN), $10);
	//module_slope->addParameter(parameter, -1);
	//printf("capacitance: %f\n",$10);
};
	


pin_parameter_slope
: YYID YYNUM YYNUM YYNUM YYNUM YYNUM YYNUM YYNUM YYNUM
{
	Parameter *parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, W2_RISE), $2);
	module_slope->addParameter(parameter, -1);
	//printf("w2 rise: %f ",$2);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, X2_RISE), $3);
	module_slope->addParameter(parameter, -1);
	//printf("x2 rise: %f ",$3);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, Y2_RISE), $4);
	module_slope->addParameter(parameter, -1);
	//printf("y2 rise: %f ",$4);
	
	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, Z2_RISE), $5);
	module_slope->addParameter(parameter, -1);
	//printf("z2 rise: %f ",$5);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, W2_FALL), $6);
	module_slope->addParameter(parameter, -1);
	//printf("w2 fall: %f ",$6);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, X2_FALL), $7);
	module_slope->addParameter(parameter, -1);
	//printf("x2 fall: %f ",$7);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, Y2_FALL), $8);
	module_slope->addParameter(parameter, -1);
	//printf("y2 fall: %f ",$8);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, Z2_FALL), $9);
	module_slope->addParameter(parameter, -1);
	//printf("z2 fall: %f\n",$9);
};







%%


void read_slope_parameters(char* filename, Lib *myLib)
{
	lib_slope = myLib;
	parameter_slope_in = fopen(filename,"rb");

	if(!parameter_slope_in)
	{
		fprintf(stderr,"Error could not open file %s\n", filename);
		exit(1);
	}

	yyparse();
	fclose(parameter_slope_in);
}

   



void parameter_slope_error(char *s)
 {
   fprintf(stderr,"%s\n",s);
 }
 

 

