%{
#include <stdio.h>
#include<stdlib.h>
#include "str_utils.h"
#include "sv_lib.h"
#include "le_leffort.h"


extern FILE *parameter_in;
extern int yyparse();
extern int yylex();
void parameter_error(char *);


Lib *lib = NULL;
Module *module = NULL;


%}


%start source

%union
 {
	char * name;
	double value;
 }

%token <name> YYBEGIN
%token <name> YYEND
%token <value> YYNUM
%token <name> YYID




%%

source
: read_parameters
{
	printf("\nparameter reading is complete\n");
}
;


read_parameters
: parameter_list
;

parameter_list
: module_parameter
| module_parameter parameter_list
;

module_parameter
: YYBEGIN '['module_name']' pin_parameter_list YYEND;

module_name
:YYID
{
	module = lib->getModule((char*)$1);
	//printf("\n\n Module name: %s\n ", module->getName());
	if (module == NULL)
	{
		printf("\n Warning: No module in Library!");
		module = new Module;
		module->setName((char*)$1);
	}
	else
		delete []$1;
};


pin_parameter_list
: pin_parameter
| pin_parameter_plus_p_theo
| pin_parameter pin_parameter_list
| pin_parameter_plus_p_theo pin_parameter_list
;


pin_parameter_plus_p_theo
: YYID YYNUM YYNUM YYNUM YYNUM YYNUM YYNUM YYNUM YYNUM
{
	//printf("pin name: %s ",$1);

	Parameter *parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, D1_PRAC_RISE), $2);
	module->addParameter(parameter, -1);
	//printf("d1 rise: %f ",$2);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, D2_PRAC_RISE), $3);
	module->addParameter(parameter, -1);
	//printf("d2 rise: %f ",$3);	

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, D1_PRAC_FALL), $4);
	module->addParameter(parameter, -1);
	//printf("d1 fall: %f ",$4);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, D2_PRAC_FALL), $5);
	module->addParameter(parameter, -1);
	//printf("d2 fall: %f ",$5);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, C_PRAC_IN), $6);
	module->addParameter(parameter, -1);
	//printf("capacitance: %f ",$6);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, C_THEO_IN), $7);
	module->addParameter(parameter, -1);
	//printf("capacitance theo: %f ",$7);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, G_THEO), $8); 
	module->addParameter(parameter, -1);
	//printf("g theo: %f ",$8);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s", P_THEO), $9); 
	module->addParameter(parameter, -1);
	//printf("p theo: %f ",$9);
};


pin_parameter
: YYID YYNUM YYNUM YYNUM YYNUM YYNUM YYNUM YYNUM
{
	//printf("pin name: %s ",$1);

	Parameter *parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, D1_PRAC_RISE), $2);
	module->addParameter(parameter, -1);
	//printf("d1 rise: %f ",$2);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, D2_PRAC_RISE), $3);
	module->addParameter(parameter, -1);
	//printf("d2 rise: %f ",$3);	

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, D1_PRAC_FALL), $4);
	module->addParameter(parameter, -1);
	//printf("d1 fall: %f ",$4);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, D2_PRAC_FALL), $5);
	module->addParameter(parameter, -1);
	//printf("d2 fall: %f ",$5);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, C_PRAC_IN), $6);
	module->addParameter(parameter, -1);
	//printf("capacitance: %f ",$6);

	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, C_THEO_IN), $7);
	module->addParameter(parameter, -1);
	//printf("capacitance theo: %f ",$7);
	
	parameter = new Parameter(TAG_UNDEFINED, name2pattern("%s%s%s", $1, SEPERATOR, G_THEO), $8); 
	module->addParameter(parameter, -1);
	//printf("g theo: %f ",$8);
};


%%


void read_parameters(char* filename, Lib *myLib)
{
	lib = myLib;
	parameter_in = fopen(filename,"rb");

	if(!parameter_in)
	{
		fprintf(stderr,"Error could not open file %s\n", filename);
		exit(1);
	}

	yyparse();
	fclose(parameter_in);
}

   



void parameter_error(char *s)
 {
   fprintf(stderr,"%s\n",s);
 }
 

 

