#ifndef _STG_NET_H_
#define _STG_NET_H_

#include "darray.h"
#include "object.h"
#include "stg_signal.h"
#include "stg_place.h"
#include "stg_transition.h"

//---------------------------------------------------------------------------
class STG : public Object
{
  public:
    SignalArray* signals;
    PlaceArray* places;
    TransitionArray* transitions;

    STG(char* name = NULL, SignalArray* signals = NULL, 
      TransitionArray* transitions = NULL, PlaceArray* places  = NULL);
    STG(const STG& stg);
    ~STG();

    Signal* STG::addSignal(char* name);
    bool addSignal(Signal* signal);
    void addSignals(SignalArray* signals);
    void removeSignal(Signal* signal);
    Signal* getSignal(char* name);
    
    Transition* addTransition(char* name);
    bool addTransition(Transition* transition);
    void addTransitions(TransitionArray* transitions);
    void removeTransition(Transition* transition);
    Transition* getTransition(char* name);

    Place* addPlace(char* name);
    bool addPlace(Place* place);
    void addPlaces(PlaceArray* places);
    void removePlace(Place* place);
    Place* getPlace(char* name);
    Place* getPlace(char* name_preset, char* name_postset);

    static void addConsumingArc(Place* place, Transition* transition);
    static void removeConsumingArc(Place* place, Transition* transition);
    static bool isConsumingArc(Place* place, Transition* transition);

    static void addProducingArc(Transition* transition, Place* place);
    static void removeProducingArc(Transition* transition, Place* place);
    static bool isProducingArc(Transition* transition, Place* place);

    static void addReadArc(Place* place, Transition* transition);
    static void removeReadArc(Place* place, Transition* transition);
    static bool isReadArc(Place* place, Transition* transition);

    void addArc(Node* from, Node* to);

    void readASTG(char* file_names);
    void writeASTG(char* file_name, bool show_implicit = false);

  protected:
    SignalHashTable* signal_hash_table;
    TransitionHashTable* transition_hash_table;
    PlaceHashTable* place_hash_table;

    void clean();
    char* genPlaceName();
    void calcIncidence();
    void calcIncidence(Transition* transition);
    void calcIncidence(Place* place);
};

#endif
