#ifndef _SV_MODULE_H_
#define _SV_MODULE_H_

#include "darray.h"
#include "hashtable.h"

class Module;
typedef darray <Module*> ModuleArray;
typedef HashTable <Module> ModuleHashTable;
typedef HashTable <Module> GenHashTable;

#include "sv_object.h"
#include "sv_parameter.h"
#include "sv_port.h"
#include "sv_instance.h"
#include "sv_lib.h"

#define NAME_MODULE_VDD                   "1"
#define NAME_MODULE_GND                   "0"

typedef darray <NetArray*> ConeArray;

//---------------------------------------------------------------------------
class Module : public VerilogObject
{
  protected:
    int depth;
    int width;
    Port* port_vdd;
    Port* port_gnd;

    void clean();

  public:
    ParameterArray* parameters;
    PortArray* ports;
    InstanceArray* instances;
    InstanceArray* users;
    Lib* lib;

    Module(int tag = TAG_UNDEFINED, char* name = NULL, ParameterArray* parameters = NULL, PortArray* ports = NULL, InstanceArray* instances = NULL, Lib* lib = NULL);
    Module(const Module &module);
    ~Module();

    void setDepth(int depth);
    int getDepth();
    void setWidth(int width);
    int getWidth();

    void addUser(Instance* user, int position = -1);
    void addUsers(InstanceArray* users);
    void removeUser(Instance* user);

    void addParameter(Parameter* parameter, int position = -1);
    void addParameters(ParameterArray* parameters);
    void removeParameter(Parameter* parameter);
    Parameter* getParameter(char* parameter_name);

    void addPort(Port* port, int position = -1);
    void addPorts(PortArray* ports);
    void removePort(Port* port);
    Port* getPort(char* port_name);
    int getPortNum(int tag_mask);
    Net* getNet(char* net_name, int net_index);
    Net* getNet(Net* net);

    void addInstance(Instance* instance, int position = -1);
    void addInstances(InstanceArray* instances);
    void removeInstance(Instance* instance);
    Instance* getInstance(char* instance_name, int instance_index = -1);

    void calcConnections();
    void sortConnections();
    void calcIncidence();
};

#endif
